#include "stdafx.h"


class DibError : public std::exception {
public:
    enum {
        unknown,        // unspecified error
        noObject        // no actual DIB exists
    };
    DibError(int code) : code(code) {}
    int code;
};


class DibPixel
{
public:
    DibPixel(void*, int, int) {}
private:
};


class DibRow
{
public:
    DibRow(void* p, int bpp, int width) : p(p), bpp(bpp), width(width) {}
    DibPixel operator[](int n) {
        if (n < 0 || n >= width)
            throw std::range_error("bad pixel index");
        switch (bpp)
        {
        case 1:
            {
                return DibPixel(((char*)p) + n/8, n%8, 1);
                break;
            }
        case 4:
            {
                return DibPixel(((char*)p) + n/2, n%2, 4);
                break;
            }
        case 8:
            {
                return DibPixel(((char*)p) + n, 0, 8);
                break;
            }
        case 16:
            {
                return DibPixel(((char*)p) + n*2, 0, 16);
                break;
            }
        case 24:
            {
                return DibPixel(((char*)p) + n*3, 0, 24);
                break;
            }
        case 32:
            {
                return DibPixel(((char*)p) + n*4, 0, 32);
                break;
            }
        /*default:
            throw ...;*/
        }
    }
private:
    void* p;
    int bpp;
    int width;
};





class Dib : boost::noncopyable
{
public:
    Dib() : bih(0), bits(0), packed(false), owned(false) {}

    ~Dib() {}

    bool isEmpty() const {
        return bih == 0;
    }

    int getWidth() const {
        ensureObj();
        return bih->biWidth;
    }

    int getHeight() const {
        ensureObj();
        return abs(bih->biWidth);
    }

    int isTopDown() const {
        ensureObj();
        return bih->biHeight < 0;
    }

    int getBpp() const {
        ensureObj();
        return bih->biBitCount;
    }

    void swap(Dib& other) {
        std::swap(bih, other.bih);
        std::swap(bits, other.bits);
        std::swap(packed, other.packed);
    }

    int getRowStride() const {
        return ((getBpp() * getWidth() + 31) / 32) * 4;
    }

    DibRow getRow(int n) {
        ensureObj();
        if (n < 0 || n >= getHeight())
            throw std::range_error("bad row index");
    }

private:
    BITMAPINFOHEADER* bih;  // 0 when no DIB exists
    void* bits;             // start of image data
    bool packed;            // if true, 'bits' is not a separate memory block
    bool owned;             // memory ownership

    void ensureObj() const {
        if (isEmpty())
            throw DibError(DibError::noObject);
    }

friend class DibRowIterator;
};
