#pragma once


namespace Graphics {;


#ifdef PNG_SUPPORT


struct PNGError {
    PNGError(std::wstring msg) : msg(msg) {}
    std::wstring msg;
};


struct ScopedFILE : boost::noncopyable {
    explicit ScopedFILE(FILE* p) : p(p) {}
    ~ScopedFILE() { if (p) fclose(p); }
    operator FILE*() { return p; }
    operator const FILE*() const { return p; }
private:
    FILE* p;
};


struct Scoped_png_write_struct: boost::noncopyable {
    explicit Scoped_png_write_struct(png_struct* p) : p(p) {}
    ~Scoped_png_write_struct() { if (p) png_destroy_write_struct(&p, 0); }
    operator png_struct*() { return p; }
    operator const png_struct*() const { return p; }
private:
    png_struct* p;
};


struct Scoped_png_info_struct: boost::noncopyable {
    explicit Scoped_png_info_struct(png_info* p, png_struct* pp) : p(p), pp(pp) {}
    ~Scoped_png_info_struct() { if (p) png_destroy_info_struct(pp, &p); }
    operator png_info*() { return p; }
    operator const png_info*() const { return p; }
private:
    png_info* p;
    png_struct* pp;
};


void writePng(std::wstring fname, const BITMAPINFOHEADER& bih);


#endif


}  // namespace Graphics
